//Author Oliver Mattis Lill?

#include<iostream>
#include<vector>
#include<string>
#define MOD 1000000007
#define MAX_N 300001
#define ACCURACY 1
#define CYCLE ('z'-'a'+1)
#define f first
#define s second
#define mp make_pair
using namespace std;


struct Modint {
	int val;

	Modint(int nval = 0) {
		val = nval;
	}

	Modint& operator+=(Modint r) {
		val = (val + r.val) % MOD;
		return *this;
	}
	Modint& operator-=(Modint r) {
		val = (val + MOD - r.val) % MOD;
		return *this;
	}
	Modint& operator*=(Modint r) {
		val = 1LL * val * r.val % MOD;
		return *this;
	}

	Modint operator+(Modint r) {return Modint(*this) += r;}
	Modint operator-(Modint r) {return Modint(*this) -= r;}
	Modint operator*(Modint r) {return Modint(*this) *= r;}
};


Modint hashField_power[ACCURACY][MAX_N];
void initHashField() {
	for(int i=0;i<ACCURACY;i++) {
		hashField_power[i][0] = 1;
		hashField_power[i][1] = 27 + 2*i;
		for(int j=2;j<MAX_N;j++)
			hashField_power[i][j] = hashField_power[i][j-1] * hashField_power[i][1];
	}
}

struct HashField {
	Modint hash[ACCURACY];

	HashField& operator+=(HashField r) {
		for(int i=0;i<ACCURACY;i++)
			hash[i] += r.hash[i];
		return *this;
	}
	HashField& operator-=(HashField r) {
		for(int i=0;i<ACCURACY;i++)
			hash[i] -= r.hash[i];
		return *this;
	}
	HashField& operator*=(Modint r) {
		for(int i=0;i<ACCURACY;i++)
			hash[i] *= r;
		return *this;
	}

	HashField operator+(HashField r) {return HashField(*this) += r;}
	HashField operator-(HashField r) {return HashField(*this) -= r;}
	HashField operator*(Modint r) {return HashField(*this) *= r;}
};

bool operator<(HashField l, HashField r) {
	for(int i=0;i<ACCURACY;i++)
		if(l.hash[i].val != r.hash[i].val)
			return l.hash[i].val < r.hash[i].val;
	return false;
}
bool operator==(HashField l, HashField r) {
	for(int i=0;i<ACCURACY;i++)
		if(l.hash[i].val != r.hash[i].val)
			return false;
	return true;
}



struct HashFieldSpan{
	vector<HashField> data;
	int n;

	void init(string s) {
		n = s.size();
		data.resize(n+1);

		for(int i=0;i<n;i++) {
			for(int j=0;j<ACCURACY;j++) {
				data[i+1].hash[j] = s[i];
				data[i+1].hash[j] += data[i].hash[j] * hashField_power[j][1];
			}
		}
	}

	HashField getSpan(int l, int r) {
		HashField ret = data[r];
		for(int i=0;i<ACCURACY;i++)
			ret.hash[i] -= data[l].hash[i] * hashField_power[i][r-l];
		return ret;
	}
};


//---------------------------------------------------------------------------------
//Now the hashfield segment tree


struct Node {
	HashField data[CYCLE];
	int lazyShift, lazySet;
	Node* lch, *rch;

	Node() {
		lch = 0, rch = 0;
		lazyShift = 0;
		lazySet = -1;
	}
};



struct Stree {
	int n;
	vector<Node> data;
	HashFieldSpan base[CYCLE];


	void setPropagate(Node* pos, int lb, int rb) {
		if(pos == 0 || pos->lazySet == -1) return;
		int mid = (lb + rb) / 2;

		if(pos->lch != 0) {
			pos->lch->lazySet = pos->lazySet;
			pos->rch->lazySet = pos->lazySet + (mid-lb);
		}
		pos->lazyShift = 0;
		for(int lev=0;lev < CYCLE;lev++)
			pos->data[lev] = base[lev].getSpan(pos->lazySet, pos->lazySet + (rb-lb) );
		pos->lazySet = -1;
	}

	//Assumes sets are performed underneath
	//Not needed

	void shiftPropagate(Node* pos, int lb, int rb) {
		if(pos == 0 || pos->lazyShift == 0) return;
		int mid = (lb + rb) / 2;

		if(pos->lch != 0) {
			setPropagate(pos->lch, lb, mid);
			setPropagate(pos->rch, mid, rb);
			pos->lch->lazyShift = (pos->lch->lazyShift + pos->lazyShift) % MOD;
			pos->rch->lazyShift += (pos->rch->lazyShift + pos->lazyShift) % MOD;
			pos->lazyShift = 0;
		}
	}


	//Composite propagation
	void propagate(Node* pos, int lb, int rb) {
		setPropagate(pos, lb, rb);
		//shiftPropagate(pos, lb, rb);
	}

	void update(Node* pos, int lb, int rb) {
		int mid = (lb + rb) / 2;

		propagate(pos, lb, rb);
		if(pos->lch != 0) {
			propagate(pos->lch, lb, mid);
			propagate(pos->rch, mid, rb);
			for(int l=0;l<CYCLE;l++)
				for(int lev=0; lev < ACCURACY; lev++)
					pos->data[l].hash[lev] =	pos->lch->data[(l+pos->lch->lazyShift)%CYCLE].hash[lev] * hashField_power[lev][rb-mid] +
					pos->rch->data[(l+pos->rch->lazyShift)%CYCLE].hash[lev];
		}

	}


	Stree(int nsize, string s) {
		n = 1;
		while(n < nsize)
			n *= 2;
		data.resize(n*2-1);

		for(auto &c : s)
			c = c - 'a';

		for(int l=0;l<CYCLE;l++) {
			base[l].init(s);
			for(int i=0;i<(int)s.size();i++)
				s[i] = (s[i]+1) % CYCLE;
		}

		vector<pair<int, int> > span(n*2-1);
		for(int i=n-1;i<2*n-1;i++) {
			span[i] = {i-(n-1), i-(n-1)+1};
		}

		for(int i=n-2;i>=0;i--) {
			data[i].lch = &data[2*i+1];
			data[i].rch = &data[2*i+2];
			span[i] = {span[2*i+1].f, span[2*i+2].s};
		}

		//Properly initialize the values
		data[0].lazySet = 0;
		for(int i=0;i<2*n-1;i++)
			setPropagate(&data[i], span[i].f, span[i].s);
	}


	//Now the actual query functionality
	void set(int l, int r, int k, Node* cur = 0, int lb = 0, int rb = -1) {
		if(rb == -1) {
			cur = &data[0];
			rb = n;
		}
		propagate(cur, lb, rb);

		if(l <= lb && rb <= r) {
			cur->lazySet = k+(lb-l);
			return;
		}

		shiftPropagate(cur, lb, rb);

		int mid = (lb + rb) / 2;
		if(l < mid)
			set(l, r, k, cur->lch, lb, mid);
		if(mid < r)
			set(l, r, k, cur->rch, mid, rb);
		update(cur, lb, rb);
	}


	void shift(int l, int r, Node* cur = 0, int lb = 0, int rb = -1) {
		if(rb == -1) {
			cur = &data[0];
			rb = n;
		}
		propagate(cur, lb, rb);

		if(l <= lb && rb <= r) {
			cur->lazyShift = (cur->lazyShift+1)%CYCLE;
			return;
		}

		int mid = (lb + rb) / 2;
		if(l < mid)
			shift(l, r, cur->lch, lb, mid);
		if(mid < r)
			shift(l, r, cur->rch, mid, rb);
		update(cur, lb, rb);
	}


	//Finally the important get functionality
	HashField get(int l, int r, int shifted = 0, Node* cur = 0, int lb = 0, int rb = -1) {
		if(rb == -1) {
			cur = &data[0];
			rb = n;
		}

		propagate(cur, lb, rb);
		shifted += cur->lazyShift;

		if(l <= lb && rb <= r)
			return cur->data[shifted];

		HashField ret;
		int mid = (lb + rb) / 2;
		if(l < mid) {
			HashField cret = get(l, r, shifted, cur->lch, lb, mid);
			for(int lev = 0; lev < ACCURACY;lev++) 
				ret.hash[lev] += cret.hash[lev] * hashField_power[lev][max(r-mid, 0)];
		}
		if(mid < r)
			ret += get(l, r, shifted, cur->rch, mid, rb);
		return ret;
	}
};



//---------------------------------------------------------------------------------
//



int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	initHashField();

	string s;
	cin >> s;

	Stree st(s.size(), s);

	int m;
	cin >> m;

	//Now answer the queries
	for(int query = 0; query < m; query++) {
		int t;
		cin >> t;

		if(t == 1) {
			int l, r, k;
			cin >> l >> r >> k;
			l--, k--;

			HashField ret1 = st.get(l, r);
			HashField ret2 = st.get(k, k+(r-l));
			if( ret1 == ret2 )
				cout << "Y\n";
			else
				cout << "N\n";
		}
		if(t == 2) {
			int l, r, k;
			cin >> l >> r >> k;
			l--, k--;

			st.set(l, r, k);
		}
		if(t == 3) {
			int l, r;
			cin >> l >> r;
			l--;

			st.shift(l, r);
		}
	}

	return 0;
}
